@@ -138,6 +138,17 @@ class SynchronizeFiles(Synchronize):
         return True
     
     def get_buttons(self):
+        exchange_site = self.root().project().get_exchange_site()
+        
+        if not exchange_site.configured.get():
+            self.message.set((
+                "<h3><font color=#D5000D>"
+                "Exchange site not configured"
+                "</font></h3>"
+                "Please configure the exchange site of this project."
+            ))
+            return ['Configure exchange site', 'Cancel']
+        
         self.message.set((
             "<h3><font color=#D66700>"
             "Synchronizing all requested files may freeze your session for a while."
@@ -151,6 +162,9 @@ class SynchronizeFiles(Synchronize):
     def run(self, button):
         if button == "Cancel":
             return
+        elif button == 'Configure exchange site':
+            exchange_site = self.root().project().get_exchange_site()
+            return self.get_result(next_action=exchange_site.configuration.oid())
         
         current_site = self.root().project().get_current_site()
         nb_waiting_init = len(current_site.get_jobs(status="WAITING"))
@@ -359,8 +373,13 @@ class ProjectSettings(flow.Object):
 
     default_files = flow.Child(DefaultFilesMap)
     non_editable_files = flow.Param("")
+    auto_upload = flow.Param('').ui(
+        tooltip='Enable <b>upload after publish</b> by default for all files matching the provided patterns')
     clean_zip_files = flow.Child(CleanZipFiles).ui(label='Clean ZIP files')
 
+    def get_auto_upload_files(self):
+        return self.auto_upload.get().replace(' ', '').split(',')
+
 
 class ExchangeSiteChoiceValue(flow.values.ChoiceValue):
 
@@ -374,18 +393,20 @@ class MultisiteConfig(flow.Object):
 
     current_site_name = flow.Computed(cached=True)
     root_dir = flow.Computed(cached=True)
+    temp_dir = flow.Computed(cached=True)
     exchange_site_name = flow.Param("default_exchange", ExchangeSiteChoiceValue).watched().ui(
         editable=False
     )
     working_sites = flow.Child(WorkingSites).ui(label="Working sites")
     exchange_sites = flow.Child(ExchangeSites).ui(label="Exchange sites")
+    sites_data = flow.Computed(cached=True).ui(hidden=True)
 
     def child_value_changed(self, child_value):
         if child_value is self.exchange_site_name:
             self.touch()
 
     def compute_child_value(self, child_value):
-        if child_value == self.root_dir:
+        if child_value is self.root_dir:
             '''
             TODO : Override by site on multisite !
             '''
@@ -396,7 +417,18 @@ class MultisiteConfig(flow.Object):
             else:
                 # Otherwise, get current site's root dir
                 root_dir = self.working_sites[self.current_site_name.get()].root_folder.get()
+            
             child_value.set(root_dir)
+        elif child_value is self.temp_dir:
+            path = self.root_dir.get()
+
+            if path is not None:
+                path = os.path.join(path, '.tmp')
+
+                if not os.path.exists(path):
+                    os.makedirs(path)
+            
+            self.temp_dir.set(path)
         elif child_value is self.current_site_name:
             if "KABARET_SITE_NAME" in os.environ:
                 site_name = os.environ["KABARET_SITE_NAME"]
@@ -411,6 +443,26 @@ class MultisiteConfig(flow.Object):
                 site_name = "default"
 
             child_value.set(site_name)
+        elif child_value is self.sites_data:
+            self.sites_data.set(self._compute_sites_data())
+    
+    def _compute_sites_data(self):
+        data = {}
+
+        for s in self.root().project().get_working_sites().mapped_items():
+            data[s.name()] = {
+                'is_default': True,
+                'is_active': s.is_active.get(),
+                'short_name': s.short_name.get()
+            }
+        for s in self.root().project().get_exchange_sites().mapped_items():
+            data[s.name()] = {
+                'is_default': True,
+                'is_active': s.is_active.get(),
+                'short_name': s.short_name.get()
+            }
+        
+        return data
 
     def summary(self):
         exchange_site = self.exchange_sites[self.exchange_site_name.get()]
@@ -464,6 +516,9 @@ class Project(flow.Object):
         if root_dir is None and alternative != None:
             root_dir = alternative
         return root_dir
+    
+    def get_temp_folder(self):
+        return self.admin.multisites.temp_dir.get()
 
     def set_user(self, username):
         project_settings_folder = self.project_settings_folder()
