@@ -10,6 +10,7 @@ import zipfile
 import fnmatch
 import re
 import traceback
+import tempfile
 
 from kabaret import flow
 from kabaret.flow_contextual_dict import ContextualView, get_contextual_dict
@@ -221,7 +222,7 @@ class MinioUploadFile(flow.Object):
         )
 
         minioClient.fput_object(
-            "testbucked",
+            exchange_site.bucket_name.get(),
             server_path,
             local_path
         )
@@ -243,11 +244,18 @@ class MinioDownloadFile(flow.Object):
             secret_key=exchange_site.server_password.get(),
             secure=True
         )
+        
+        tmp_path = self.root().project().get_temp_folder()
+
+        if tmp_path is not None:
+            tmp_name = os.path.splitext(os.path.basename(local_path))[0]
+            tmp_path = os.path.join(tmp_path, tmp_name)
 
         minioClient.fget_object(
-            "testbucked",
+            exchange_site.bucket_name.get(),
             server_path,
-            local_path
+            local_path,
+            tmp_file_path=tmp_path
         )
 
 
@@ -741,11 +749,7 @@ class Site(flow.Object):
     short_name = flow.Param("")
     description = flow.Param("")
     # Define conditions required for a site to be considered as correctly configured
-    configured = flow.Computed(
-        cached=True,
-        store_value=True,
-        computed_value_type=ComputedBoolValue
-    )
+    configured = flow.Computed()
     configuration = flow.Child(ConfigureSite)
 
     is_active = flow.BoolParam(True)
@@ -759,6 +763,9 @@ class SiteMap(flow.Map):
 
     create_site = flow.Child(CreateSite)
 
+    _short_names = flow.HashParam()
+    _active_site_names = flow.OrderedStringSetParam()
+
     @classmethod
     def mapped_type(cls):
         return Site
@@ -770,33 +777,55 @@ class SiteMap(flow.Map):
         return ["Site"]
     
     def short_names(self):
-        return [s.short_name.get() for s in self.mapped_items()]
+        short_names = []
+        
+        for name in self.mapped_names():
+            short_names.append(self.get_short_name(name))
+        
+        return short_names
+    
+    def get_short_name(self, site_name):
+        return self._short_names.get_key(site_name)
+    
+    def update_active_site_names(self):
+        self._active_site_names.revert_to_default()
+        i = 0
+
+        for s in self.mapped_items():
+            if s.is_active.get():
+                self._active_site_names.add(s.name(), i)
+            
+            i += 1
+    
+    def update_site_short_names(self):
+        self._short_names.revert_to_default()
+
+        for s in self.mapped_items():
+            self._short_names.set_key(s.name(), s.short_name.get())
     
     def get_site_names(self, use_custom_order=False, active_only=False, short_names=False):
-        if not use_custom_order:
-            sites = self.mapped_items()
-        else:
+        sites_data = self.root().project().admin.multisites.sites_data.get()
+        
+        if use_custom_order:
             current_site = self.root().project().get_current_site()
-            unordered_names = self.mapped_names()
-            sites = []
-            
-            for name in current_site.get_ordered_site_names():
-                if self.has_mapped_name(name):
-                    sites.append(self.get_mapped(name))
-                    unordered_names.remove(name)
-            
-            for name in unordered_names:
-                sites.append(self.get_mapped(name))
-
-        if active_only:
-            sites = [s for s in sites if s.is_active.get()]
-        
-        if short_names:
-            names = [s.short_name.get() for s in sites]
+            names = current_site.ordered_site_names.get()
         else:
-            names = [s.name() for s in sites]
+            names = self.mapped_names()
+
+        site_names = []
+
+        for name in names:
+            data = sites_data[name]
+
+            if active_only and not data['is_active']:
+                continue
+            
+            if short_names:
+                site_names.append(data['short_name'])
+            else:
+                site_names.append(name)
         
-        return names
+        return site_names
     
     def _get_mapped_item_type(self, mapped_name):
         if mapped_name == "default":
@@ -1009,7 +1038,10 @@ class WorkingSite(Site):
     root_linux_folder = flow.Param()
     root_darwin_folder = flow.Param()
     
-    ordered_site_names = flow.Param('')
+    ordered_site_names = flow.Computed(cached=True)
+    custom_site_order = flow.Param('').watched().ui(
+        label='Custom order',
+        tooltip='Manage order in which sites are listed in the interface')
     
     sync_dl_max_connections = flow.IntParam(1)
     sync_up_max_connections = flow.IntParam(1)
@@ -1051,12 +1083,26 @@ class WorkingSite(Site):
                     or self.root_darwin_folder.get()
                 )
             )
+        elif child_value is self.ordered_site_names:
+            self.ordered_site_names.set(self._compute_ordered_site_names())
     
-    def get_ordered_site_names(self):
-        names_as_string = self.ordered_site_names.get()
-        names_as_string = names_as_string.replace(' ','')
+    def child_value_changed(self, child_value):
+        if child_value is self.custom_site_order:
+            self.ordered_site_names.touch()
+    
+    def _compute_ordered_site_names(self):
+        names_as_string = self.custom_site_order.get()
+        names = names_as_string.replace(' ','').split(',')
         
-        return names_as_string.split(',')
+        if names and names[0] == '':
+            names = []
+        
+        unordered_names = self._site_map.mapped_names()
+
+        for n in names:
+            unordered_names.remove(n)
+        
+        return names + unordered_names
     
     def get_queue(self):
         return self.queue.job_list
@@ -1119,18 +1165,21 @@ class ConfigureExchangeSite(ConfigureSite):
     server_url = flow.SessionParam("http://")
     server_login = flow.SessionParam("")
     server_password = flow.SessionParam("").ui(editor='password')
+    bucket_name = flow.SessionParam('')
 
     def _fill_action_fields(self):
         super(ConfigureExchangeSite, self)._fill_action_fields()
         self.server_url.set(self._site.server_url.get())
         self.server_login.set(self._site.server_login.get())
         self.server_password.set(self._site.server_password.get())
+        self.bucket_name.set(self._site.bucket_name.get())
 
     def _configure_site(self, site):
         super(ConfigureExchangeSite, self)._configure_site(site)
         site.server_url.set(self.server_url.get())
         site.server_login.set(self.server_login.get())
         site.server_password.set(self.server_password.get())
+        site.bucket_name.set(self.bucket_name.get())
 
 
 class ExchangeSite(Site):
@@ -1140,6 +1189,7 @@ class ExchangeSite(Site):
     server_url = flow.Param("")
     server_login = flow.Param("")
     server_password = flow.Param("").ui(editor='password')
+    bucket_name = flow.Param('')
 
     configuration = flow.Child(ConfigureExchangeSite)
 
@@ -1150,6 +1200,7 @@ class ExchangeSite(Site):
                     self.server_url.get()
                     and self.server_login.get()
                     and self.server_password.get()
+                    and self.bucket_name.get()
                 )
             )
 
@@ -1182,17 +1233,6 @@ class ExchangeSites(SiteMap):
 class SyncSiteStatus(flow.Object):
     status = flow.Param("NotAvailable", StaticSiteSyncStatusChoices)
 
-    def get_short_name(self):
-        working_sites = self.root().project().get_working_sites()
-        exchange_sites = self.root().project().get_exchange_sites()
-
-        if working_sites.has_mapped_name(self.name()):
-            site = working_sites[self.name()]
-        else:
-            site = exchange_sites[self.name()]
-
-        return site.short_name.get()
-
 
 class SyncMap(flow.DynamicMap):
     version = flow.Parent()
@@ -1202,20 +1242,20 @@ class SyncMap(flow.DynamicMap):
         return SyncSiteStatus
 
     def mapped_names(self, page_num=0, page_size=None):
-        working_sites = self.root().project().get_working_sites()
-        exchange_sites = self.root().project().get_exchange_sites()
-        
-        return working_sites.mapped_names() + exchange_sites.mapped_names()
+        sites_data = self.root().project().admin.multisites.sites_data.get()
+        return sites_data.keys()
 
     def columns(self):
         return ['Name', 'Status']
 
     def _fill_row_cells(self, row, item):
         row['Status'] = item.status.get()
-        row['Name'] = item.name()
+        name = item.name()
 
-        if item.name() == self.version.site.get():
-            row['Name'] += " (S)"
+        if name == self.version.site.get():
+            name += " (S)"
+        
+        row['Name'] = name
 
 
 class RequestedRevisions(flow.DynamicMap):
