@@ -7,6 +7,8 @@ from kabaret.subprocess_manager import SubprocessManager, LauncherToolBar
 
 from libreflow.utils.kabaret.jobs.jobs_node import JobsNodeSession
 
+from .utils.kabaret.jobs.jobs_actor import Jobs
+
 
 class BaseGUISession(gui.KabaretStandaloneGUISession):
     def register_view_types(self):
@@ -33,6 +35,7 @@ class BaseCLISession(KabaretSession):
 
         # Configure SubprocessManager
         SubprocessManager(self)
+        Jobs(self)
 
 
 class JobsNodeSession(JobsNodeSession):
